# $Id: Config.py,v 1.4 2005/03/27 17:06:42 asc Exp $

__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/ImageWidget/"
__version__    = "1.0"
__cvsversion__ = "$Revision: 1.4 $"
__date__       = "$Date: 2005/03/27 17:06:42 $"
__copyright__  = "Copyright (c) 2004-2005 Aaron Straup Cope. GNU GPL 2"

from shutil  import copy
import os

from rdflib.constants   import TYPE
from rdflib.Literal	import Literal
from rdflib.Namespace   import Namespace
from rdflib.TripleStore import TripleStore
from rdflib.URIRef	import URIRef
from rdflib.BNode       import BNode

# I don't really understand why I need to do
# this but py2app standalones blow their brains
# out without it...

from rdflib.syntax.serializers.NTSerializer import NTSerializer
from rdflib.syntax.serializers.XMLSerializer import XMLSerializer
from rdflib.syntax.parsers.NTParser import NTParser
from rdflib.syntax.parsers.RDFXMLParser import RDFXMLParser

from ImageWidget.Error import ImageWidgetError
from ImageWidget.Constants import *

LOCAL_RDFNS_RI = Namespace(IMAGEWIDGET_NAMESPACE)

class ImageWidgetConfig:

    def __init__ (self,ns_app,file) :

        self.__store__     = TripleStore()
        self.__cfg__       = None
        self.__lastread__  = 0
        
	if not os.path.isfile(file) :

	    APP   = Namespace(ns_app)
	    CFG   = Namespace(LOCAL_RDFNS_RI)
	    prefs = BNode()

	    self.__store__.add((prefs,
				TYPE,
				APP[IMAGEWIDGET_CFG_PREFERENCES]))

	    self.__store__.add((prefs,
				CFG[IMAGEWIDGET_CFG_LOGGING],
				Literal(IMAGEWIDGET_DEFAULT_LOGGING)))

	    self.__store__.add((prefs,
				CFG[IMAGEWIDGET_CFG_POSX],
				Literal(IMAGEWIDGET_DEFAULT_POSX)))

	    self.__store__.add((prefs,
				CFG[IMAGEWIDGET_CFG_POSY],
				Literal(IMAGEWIDGET_DEFAULT_POSY)))

	    self.__store__.add((prefs,
				CFG[IMAGEWIDGET_CFG_MAXWIDTH],
				Literal(IMAGEWIDGET_DEFAULT_SZX)))

	    self.__store__.add((prefs,
				CFG[IMAGEWIDGET_CFG_MAXHEIGHT],
				Literal(IMAGEWIDGET_DEFAULT_SZY)))

            self.__store__.save(file)

        self.__cfg__ = file
	self.__ns__  = Namespace(ns_app)

	self.Load()

	try :
	    self.Load()
	except Exception, e :
	    raise ImageWidgetError(IMAGEWIDGET_ERR_LOADCONFIG,e)

    #
    
    def __del__(self):
	self.Save()

    #

    def nsApp (self) :
	return self.__ns__	

    def nsCfg (self ) :
	return LOCAL_RDFNS_RI
    
    def nsCustom (self,uri) :
	return Namespace(uri)

    #

    def Load (self) :

        if self.__cfg__ == None:
            raise ImageWidget.Error(IMAGEWIDGET_ERR_NOCONFIG)

        #
        
        modified = os.stat(self.__cfg__)[8]
        
        if self.__lastread__ >= modified:
            return True

        #

        try :
            self.__store__.load(self.__cfg__)
        except Exception, e:
            raise ImageWidget.Error(IMAGEWIDGET_ERR_LOADCONFIG,e)
        
        #

	prefs = self.nsApp()[IMAGEWIDGET_CFG_PREFS]
	gen   = self.__store__.subjects(TYPE,prefs)

	try :
	    self.__prefs__ = gen.next()
	except StopIteration, e :
	    raise ImageWidget.Error(IMAGEWIDGET_ERR_STOPCONFIG,e)
	except Exception, e :
	    raise ImageWidget.Error(IMAGEWIDGET_ERR_READCONFIG,e)

	self.__lastread__ = modified

    #
    
    def Save (self) :
	try :
	    self.Write()
        except Exception, e :
	    err = wx.MessageDialog(self, e,
				   IMAGEWIDGET_CFG_ERROR_TITLE,
				   wx.OK);
	    
	    err.ShowModal()
	    err.Destroy()					

    def Write (self) :
        
        file   = os.path.abspath(self.__cfg__)
        backup = file + ".bak"

	#

	if os.path.isfile(file) :
	    try:
		copy(file,backup)
		os.chmod(backup,0600)
	    except Exception, e:
		raise ImageWidget.Error(IMAGEWIDGET_ERR_BACKUPCONFIG,e)

	#

        try:
            self.__store__.save(file)
	    os.chmod(file,0600)

            return True
        except Exception, e:
            raise ImageWidget.Error(IMAGEWIDGET_ERR_SAVECONFIG,e)

    #

    def GetPref (self,predicate) :

	uri = URIRef(predicate)
	gen = self.__store__.objects(self.__prefs__,uri)

	try :
	    res = gen.next()
	except StopIteration, e :
	    raise ImageWidget.Error(IMAGEWIDGET_ERR_STOPPREF,e)
	except Exception, e :
	    str_err = "%s : %s" % (IMAGEWIDGET_ERR_READPREF,predicate)
	    raise ImageWidget.Error(str_err,e)

	return res

    #

    def SetPref (self,predicate,value) :
	object = Literal(value)

        self.__store__.remove((self.__prefs__,predicate,None))  
        self.__store__.add((self.__prefs__,predicate,object)) 

    #

    def EnsureStubProperty (self,predicate) :
	try :
	    self.GetPref(predicate)
	except :
	    self.SetPref(predicate,"")
